@@ -4,7 +4,7 @@
  * 
  * copyright (c) 2001-7, Karl W Broman
  * 
- * last modified Jun, 2007
+ * last modified Oct, 2007
  * first written Feb, 2001
  *
  * Licensed under the GNU General Public License version 2 (June, 1991)
@@ -15,7 +15,7 @@
  *           nrec_4way2, calc_genoprob_4way, calc_genoprob_special_4way, sim_geno_4way, 
  *           est_map_4way, argmax_geno_4way, errorlod_4way, 
  *           calc_errorlod_4way, nrec2_4way, logprec_4way, est_rf_4way
- *           calc_pairprob_4way
+ *           calc_pairprob_4way, marker_loglik_4way
  *
  * These are the init, emit, and step functions plus
  * all of the hmm wrappers for the "4-way" cross (autosomal data)
@@ -94,6 +94,27 @@ double emit_4way(int obs_gen, int true_gen, double error_prob)
     case 2: case 3: return(log(1.0-2.0*error_prob/3.0));
     case 1: case 4: return(log(2.0*error_prob/3.0));
     }
+  case 11:
+    switch(true_gen) {
+    case 1: return(log(error_prob));
+    case 2: case 3: case 4: return(log(1.0-error_prob/3.0));
+    }
+  case 12:
+    switch(true_gen) {
+    case 2: return(log(error_prob));
+    case 1: case 3: case 4: return(log(1.0-error_prob/3.0));
+    }
+  case 13:
+    switch(true_gen) {
+    case 3: return(log(error_prob));
+    case 1: case 2: case 4: return(log(1.0-error_prob/3.0));
+    }
+  case 14:
+    switch(true_gen) {
+    case 4: return(log(error_prob));
+    case 1: case 2: case 3: return(log(1.0-error_prob/3.0));
+    }
+
   }
   return(0.0); /* shouldn't get here */
 }
@@ -257,11 +278,16 @@ double errorlod_4way(int obs, double *prob, double error_prob)
   case 8: p=(prob[2]+prob[3]); break;
   case 9: p=(prob[0]+prob[3]); break;
   case 10: p=(prob[1]+prob[2]); break;
+  case 11: p=1.0-prob[0]; break;
+  case 12: p=1.0-prob[1]; break;
+  case 13: p=1.0-prob[2]; break;
+  case 14: p=1.0-prob[3]; break;
   }
   
   p = (1.0-p)/p;
 
-  if(obs>4) p *= (1.0-2.0*error_prob/3.0)/(2.0*error_prob/3.0);
+  if(obs>10) p *= (1.0-error_prob/3.0)/error_prob;
+  else if(obs>4 && obs<11) p *= (1.0-2.0*error_prob/3.0)/(2.0*error_prob/3.0);
   else p *= (1.0-error_prob)/error_prob;
 
   if(p < TOL) return(-12.0);
@@ -295,8 +321,11 @@ double nrec2_4way(int obs1, int obs2, double rf)
     case 4: return(2.0);
     case 5: case 7: return(rf);
     case 6: case 8: return(1.0+rf);
-    case 9: return(2*rf*rf/(rf*rf+(1.0-rf)*(1.0-rf)));
+    case 9: return(2.0*rf*rf/(rf*rf+(1.0-rf)*(1.0-rf)));
     case 10: return(1.0);
+    case 11: return(2.0*rf/(1.0-(1.0-rf)*(1.0-rf)));
+    case 12: case 13: return(rf*(1.0+rf)/(1.0-rf*(1.0-rf)));
+    case 14: return(2.0*rf*(1.0-rf)/(1.0-rf*rf));
     }
   case 2:
     switch(obs2) {
@@ -306,7 +335,10 @@ double nrec2_4way(int obs1, int obs2, double rf)
     case 5: case 8: return(1.0+rf);
     case 6: case 7: return(rf);
     case 9: return(1.0);
-    case 10: return(2*rf*rf/(rf*rf+(1.0-rf)*(1.0-rf)));
+    case 10: return(2.0*rf*rf/(rf*rf+(1.0-rf)*(1.0-rf)));
+    case 12: return(2.0*rf/(1.0-(1.0-rf)*(1.0-rf)));
+    case 11: case 14: return(rf*(1.0+rf)/(1.0-rf*(1.0-rf)));
+    case 13: return(2.0*rf*(1.0-rf)/(1.0-rf*rf));
     }
   case 3:
     switch(obs2) {
@@ -315,41 +347,86 @@ double nrec2_4way(int obs1, int obs2, double rf)
     case 5: case 8: return(rf);
     case 6: case 7: return(1.0+rf);
     case 9: return(1.0);
-    case 10: return(2*rf*rf/(rf*rf+(1.0-rf)*(1.0-rf)));
+    case 10: return(2.0*rf*rf/(rf*rf+(1.0-rf)*(1.0-rf)));
+    case 13: return(2.0*rf/(1.0-(1.0-rf)*(1.0-rf)));
+    case 11: case 14: return(rf*(1.0+rf)/(1.0-rf*(1.0-rf)));
+    case 12: return(2.0*rf*(1.0-rf)/(1.0-rf*rf));
     }
   case 4: 
     switch(obs2) {
     case 4: return(0.0);
     case 5: case 7: return(1.0+rf);
     case 6: case 8: return(rf);
-    case 9: return(2*rf*rf/(rf*rf+(1.0-rf)*(1.0-rf)));
+    case 9: return(2.0*rf*rf/(rf*rf+(1.0-rf)*(1.0-rf)));
     case 10: return(1.0);
+    case 14: return(2.0*rf/(1.0-(1.0-rf)*(1.0-rf)));
+    case 12: case 13: return(rf*(1.0+rf)/(1.0-rf*(1.0-rf)));
+    case 11: return(2.0*rf*(1.0-rf)/(1.0-rf*rf));
     }
   case 5: 
     switch(obs2) {
     case 5: return(rf);
     case 6: return(1.0+rf);
     case 7: case 8: case 9: case 10: return(2.0*rf);
+    case 11: case 13: return(rf*(3.0+rf)/(1.0+rf));
+    case 12: case 14: return(rf*(3.0-rf)/(2.0-rf));
     }
   case 6: 
     switch(obs2) {
     case 6: return(rf);
     case 7: case 8: case 9: case 10: return(2.0*rf);
+    case 12: case 14: return(rf*(3.0+rf)/(1.0+rf));
+    case 11: case 13: return(rf*(3.0-rf)/(2.0-rf));
     }
   case 7:
     switch(obs2) {
     case 7: return(rf);
     case 8: return(1.0+rf);
     case 9: case 10: return(2.0*rf);
+    case 11: case 12: return(rf*(3.0+rf)/(1.0+rf));
+    case 13: case 14: return(rf*(3.0-rf)/(2.0-rf));
     }
   case 8:
     switch(obs2) {
     case 8: return(rf);
     case 9: case 10: return(2.0*rf);
+    case 13: case 14: return(rf*(3.0+rf)/(1.0+rf));
+    case 11: case 12: return(rf*(3.0-rf)/(2.0-rf));
+    }
+  case 9: 
+    switch(obs2) {
+    case 9: return(2.0*rf*rf/(rf*rf+(1.0-rf)*(1.0-rf)));
+    case 10: return(1.0);
+    case 11: case 14: return(2.0*rf*(2.0-rf)/(2.0-(1.0-rf)*(1.0-rf)-rf*rf));
+    case 12: case 13: return(rf*(1.0+rf)/(1.0-rf*(1.0-rf)));
+    }
+  case 10: 
+    switch(obs2) {
+    case 10: return(1.0);
+    case 12: case 13: return(2.0*rf*(2.0-rf)/(2.0-(1.0-rf)*(1.0-rf)-rf*rf));
+    case 11: case 14: return(rf*(1.0+rf)/(1.0-rf*(1.0-rf)));
+    }
+  case 11: 
+    switch(obs2) {
+    case 11: return(4.0*rf/(2.0+(1.0-rf)*(1.0-rf)));
+    case 12: case 13: return(rf*(5.0-rf)/(2.0+rf*(1.0-rf)));
+    case 14: return(2.0*rf*(2.0+rf)/(2.0+rf*rf));
+    }
+  case 12: 
+    switch(obs2) {
+    case 12: return(4.0*rf/(2.0+(1.0-rf)*(1.0-rf)));
+    case 14: return(rf*(5.0-rf)/(2.0+rf*(1.0-rf)));
+    case 13: return(2.0*rf*(2.0+rf)/(2.0+rf*rf));
+    }
+  case 13: 
+    switch(obs2) {
+    case 13: return(4.0*rf/(2.0+(1.0-rf)*(1.0-rf)));
+    case 14: return(rf*(5.0-rf)/(2.0+rf*(1.0-rf)));
+    }
+  case 14: 
+    switch(obs2) {
+    case 14: return(4.0*rf/(2.0+(1.0-rf)*(1.0-rf)));
     }
-  case 9: case 10: 
-    if(obs1==obs2) return(2*rf*rf/(rf*rf+(1.0-rf)*(1.0-rf)));
-    else return(1.0);
   }
   return(log(-1.0)); /* shouldn't get here */
 }
@@ -375,6 +452,9 @@ double logprec_4way(int obs1, int obs2, double rf)
     case 6: case 8: return(log(rf));
     case 9: return(log(rf*rf+(1.0-rf)*(1.0-rf)));
     case 10: return(log(2.0)+log(rf)+log(1.0-rf));
+    case 11: return(log(1.0-(1.0-rf)*(1.0-rf)));
+    case 12: case 13: return(log(1.0 - rf*(1.0-rf)));
+    case 14: return(log(1.0-rf*rf));
     }
   case 2:
     switch(obs2) {
@@ -385,6 +465,9 @@ double logprec_4way(int obs1, int obs2, double rf)
     case 6: case 7: return(log(1.0-rf));
     case 9: return(log(2.0)+log(rf)+log(1.0-rf));
     case 10: return(log(rf*rf+(1.0-rf)*(1.0-rf)));
+    case 12: return(log(1.0-(1.0-rf)*(1.0-rf)));
+    case 11: case 14: return(log(1.0 - rf*(1.0-rf)));
+    case 13: return(log(1.0-rf*rf));
     }
   case 3:
     switch(obs2) {
@@ -394,6 +477,9 @@ double logprec_4way(int obs1, int obs2, double rf)
     case 6: case 7: return(log(rf));
     case 9: return(log(2.0)+log(rf)+log(1.0-rf));
     case 10: return(log(rf*rf+(1.0-rf)*(1.0-rf)));
+    case 13: return(log(1.0-(1.0-rf)*(1.0-rf)));
+    case 11: case 14: return(log(1.0 - rf*(1.0-rf)));
+    case 12: return(log(1.0-rf*rf));
     }
   case 4: 
     switch(obs2) {
@@ -402,33 +488,76 @@ double logprec_4way(int obs1, int obs2, double rf)
     case 6: case 8: return(log(1.0-rf));
     case 9: return(log(rf*rf+(1.0-rf)*(1.0-rf)));
     case 10: return(log(2.0)+log(rf)+log(1.0-rf));
+    case 14: return(log(1.0-(1.0-rf)*(1.0-rf)));
+    case 12: case 13: return(log(1.0 - rf*(1.0-rf)));
+    case 11: return(log(1.0-rf*rf));
     }
   case 5: 
     switch(obs2) {
     case 5: return(log(2.0)+log(1.0-rf));
     case 6: return(log(2.0)+log(rf));
     case 7: case 8: case 9: case 10: return(0.0);
+    case 11: case 13: return(log(1.0+rf));
+    case 12: case 14: return(log(2.0-rf));
     }
   case 6: 
     switch(obs2) {
     case 6: return(log(2.0)+log(1.0-rf));
     case 7: case 8: case 9: case 10: return(0.0);
+    case 12: case 14: return(log(1.0+rf));
+    case 11: case 13: return(log(2.0-rf));
     }
   case 7:
     switch(obs2) {
     case 7: return(log(2.0)+log(1.0-rf));
     case 8: return(log(2.0)+log(rf));
     case 9: case 10: return(0.0);
+    case 11: case 12: return(log(1.0+rf));
+    case 13: case 14: return(log(2.0-rf));
     }
   case 8:
     switch(obs2) {
     case 8: return(log(2.0)+log(1.0-rf));
     case 9: case 10: return(0.0);
+    case 13: case 14: return(log(1.0+rf));
+    case 11: case 12: return(log(2.0-rf));
+    }
+  case 9: 
+    switch(obs2) {
+    case 9: return(log(2.0)+log(rf*rf+(1.0-rf)*(1.0-rf)));
+    case 10:return(log(4.0)+log(rf)+log(1.0-rf));
+    case 11: case 14: return(log(2.0-(1.0-rf)*(1.0-rf)-rf*rf));
+    case 12: case 13: return(log(2.0-2.0*rf*(1.0-rf)));
+    }
+  case 10: 
+    switch(obs2) {
+    case 10: return(log(2.0)+log(rf*rf+(1.0-rf)*(1.0-rf)));
+    case 12: case 13: return(log(2.0-(1.0-rf)*(1.0-rf)-rf*rf));
+    case 11: case 14: return(log(2.0-2.0*rf*(1.0-rf)));
+    }
+  case 11: 
+    switch(obs2) {
+    case 11: return(log(2.0+(1.0-rf)*(1.0-rf)));
+    case 12: case 13: return(log(2.0+rf*(1.0-rf)));
+    case 14: return(log(2.0+rf*rf));
+    }
+  case 12: 
+    switch(obs2) {
+    case 12: return(log(2.0+(1.0-rf)*(1.0-rf)));
+    case 13: return(log(2.0+rf*rf));
+    case 14: return(log(2.0+rf*(1.0-rf)));
+    }
+  case 13: 
+    switch(obs2) {
+    case 13: return(log(2.0+(1.0-rf)*(1.0-rf)));
+    case 14: return(log(2.0+rf*(1.0-rf)));
+    }
+  case 14: 
+    switch(obs2) {
+    case 14: return(log(2.0+(1.0-rf)*(1.0-rf)));
     }
-  case 9: case 10: 
-    if(obs1==obs2) return(log(2.0)+log(rf*rf+(1.0-rf)*(1.0-rf)));
-    else return(log(4.0)+log(rf)+log(1.0-rf));
   }
+    
   return(log(-1.0)); /* shouldn't get here */
 }
 
@@ -447,4 +576,13 @@ void calc_pairprob_4way(int *n_ind, int *n_mar, int *geno,
 		pairprob, init_4way, emit_4way, step_4way);
 }
 
+void marker_loglik_4way(int *n_ind, int *geno,
+			double *error_prob, double *loglik)
+{
+  marker_loglik(*n_ind, 4, geno, *error_prob, init_4way, emit_4way,
+		loglik);
+}
+
+
+
 /* end of hmm_4way.c */
